==> hermes-2.5.1.36.tar.gz: hermes-2.5.1.36/spec/apis/dmo/housing_offer_expiration_settings_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    department: 'String',
    max_applicants: 'Integer',
    expiry_days: 'Integer',
    minimal_publication_days: 'Integer',
    lessor_id: 'String'
  }

  ACCEPTABLE_PARAMS = %w[
    department
    max_applicants
    expiry_days
    minimal_publication_days
    lessor_id
    created_at
    updated_at
  ]

  RSpec.shared_examples 'admin only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::HousingOfferExpirationSettingsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:roles) { %w[reservataire bailleur entreprise demandeur Manager Supervisor] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }

    let(:setting) { create :expiration_setting }
    let(:params) do
      { attributes: { max_applicants: 10, department: '75' } }
    end

    before { authenticated_header(request, responsible) }

    describe 'GET index' do
      let!(:settings) { create_list :expiration_setting, 5 }
      subject { get 'index' }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      subject { get 'show', { id: setting.id } }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq setting.id.to_s
        expect(json['data']['attributes']['max_applicants']).to eq setting.max_applicants
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates an expiration setting' do
        expect do
          post 'create', { data: params }
        end.to change { Hermes::HousingOfferExpirationSetting.count }.by 1
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['department']).to eq '75'
        expect(json['data']['attributes']['max_applicants']).to eq 10
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: setting.id, data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end

      it 'updates the setting' do
        expect do
          subject
        end.to change { setting.reload.department }.from('34').to '75'
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '75'
      end
    end

    describe 'DELETE destroy' do
      let(:rand_admin) { create :user, roles: ['Administrator'] }
      let!(:owned_setting) { create :expiration_setting, user_id: responsible.id }

      subject { delete 'destroy', { id: owned_setting.id.to_s } }

      it_behaves_like 'admin only'

      context 'for owner' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end

        it 'destroys the setting' do
          expect do
            subject
          end.to change { Hermes::HousingOfferExpirationSetting.count }.by -1
        end
      end

      context 'for another admin' do
        before { authenticated_header(request, rand_admin) }

        it 'returns error' do
          subject
          expect(response).to have_http_status 404
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingOfferExpirationSettingsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS = 'API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/housing_offer_expiration_settings' do
        post 'Create expiration_setting' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = { type: nice_type(val) }
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'expiration_setting created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes({})
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offer_expiration_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings' do
        get 'Retrieves all expiration_settings' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'expiration_settings found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:settings) { create_list(:expiration_setting, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { settings }
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        get 'Retrieves a single expiration_setting by id' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'expiration_setting found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:expiration_setting) { create(:expiration_setting) }
            let(:id) { expiration_setting.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { expiration_setting }
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        put 'edit expiration_setting' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'expiration_setting updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:expiration_setting) { create(:expiration_setting) }
            let(:id) { expiration_setting.id }
            let(:params) do
              jsonapi_attributes(
                 { lessor_id: 'String' }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offer_expiration_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        delete 'Delete a single expiration setting by id' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'expiration_setting deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:expiration_setting, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end

==> hermes-4.1.0.5.tar.gz: hermes-4.1.0.5/spec/apis/dmo/housing_offer_expiration_settings_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    department: 'String',
    max_applicants: 'Integer',
    expiry_days: 'Integer',
    minimal_publication_days: 'Integer',
    lessor_id: 'String'
  }

  ACCEPTABLE_PARAMS = %w[
    department
    max_applicants
    expiry_days
    minimal_publication_days
    lessor_id
    created_at
    updated_at
  ]

  RSpec.shared_examples 'admin only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::HousingOfferExpirationSettingsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:roles) { %w[bailleur entreprise demandeur Manager Supervisor] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }

    let(:setting) { create :expiration_setting }
    let(:params) do
      { attributes: { max_applicants: 10, department: '75' } }
    end

    before { authenticated_header(request, responsible) }

    describe 'GET index' do
      let!(:settings) { create_list :expiration_setting, 5 }
      subject { get 'index' }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      subject { get 'show', { id: setting.id } }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq setting.id.to_s
        expect(json['data']['attributes']['max_applicants']).to eq setting.max_applicants
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates an expiration setting' do
        expect do
          post 'create', { data: params }
        end.to change { Hermes::HousingOfferExpirationSetting.count }.by 1
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['department']).to eq '75'
        expect(json['data']['attributes']['max_applicants']).to eq 10
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: setting.id, data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end

      it 'updates the setting' do
        expect do
          subject
        end.to change { setting.reload.department }.from('34').to '75'
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '75'
      end
    end

    describe 'DELETE destroy' do
      let(:rand_admin) { create :user, roles: ['Administrator'] }
      let!(:owned_setting) { create :expiration_setting, user_id: responsible.id }

      subject { delete 'destroy', { id: owned_setting.id.to_s } }

      it_behaves_like 'admin only'

      context 'for owner' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end

        it 'destroys the setting' do
          expect do
            subject
          end.to change { Hermes::HousingOfferExpirationSetting.count }.by -1
        end
      end

      context 'for another admin' do
        before { authenticated_header(request, rand_admin) }

        it 'returns error' do
          subject
          expect(response).to have_http_status 404
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingOfferExpirationSettingsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS = 'API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/housing_offer_expiration_settings' do
        post 'Create expiration_setting' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = { type: nice_type(val) }
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'expiration_setting created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes({})
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offer_expiration_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings' do
        get 'Retrieves all expiration_settings' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'expiration_settings found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:settings) { create_list(:expiration_setting, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { settings }
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        get 'Retrieves a single expiration_setting by id' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'expiration_setting found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:expiration_setting) { create(:expiration_setting) }
            let(:id) { expiration_setting.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { expiration_setting }
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        put 'edit expiration_setting' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'expiration_setting updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:expiration_setting) { create(:expiration_setting) }
            let(:id) { expiration_setting.id }
            let(:params) do
              jsonapi_attributes(
                 { lessor_id: 'String' }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_offer_expiration_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_offer_expiration_settings/{id}' do
        delete 'Delete a single expiration setting by id' do
          tags TAGS_API_DMO_HOUSING_OFFER_EXPIRATION_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'expiration_setting deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:expiration_setting, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end
